
/*  macosd - a pbbuttonsd client to display the pbbuttonsd events
 *           (and later maybe some other events - like Evkeyd, too)
 *  Copyright (C) 2003 Ren Rebe <rene@rocklinux.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <memory> // auto_ptr

#include "config.h"

#include "PBB.hh"
#include "ArgumentList.hh"

#if WITHXOSD != 0
#include "PBBXosd.hh"
#endif

#if WITHEVAS != 0
#include "PBBEvas.hh"
#endif

using namespace Utility;

int main (int argc, char** argv)
{
  using namespace std;
  
  ArgumentList arglist;
  
  const std::string default_theme =
#if WITHEVAS !=0
    "CleanOSX"
#else
    "classic"
#endif
    ;
  
  // setup the argument list
  Argument<std::string> arg_theme ("t", "theme", "select the theme to be used",
				   default_theme, 0, 1);
  Argument<bool> arg_list ("l", "list","list the available themes");
  Argument<bool> arg_help ("h", "help", "display this help text and exit");
  Argument<bool> arg_nodemo ("n", "nodemo",
			     "option to skip demo intro (intentionally hidden)",
			     0, 0, false, false);
  
  arglist.Add (&arg_theme);
  arglist.Add (&arg_list);
  arglist.Add (&arg_help);
  arglist.Add (&arg_nodemo);

  // parse the specified argument list - and maybe output the Usage
  if (!arglist.Read (argc, argv) || arg_help.Get() == true)
    {
      std::cout << "MacOSD " << VERSION
                <<  " - Copyright 2003, 2004 by Ren Rebe" << std::endl
		<< "Useage:" << std::endl;
      
      arglist.Usage (std::cout);
      return 1;
    }
  
  // instantiate the view to let them gather themes ...
  PBBRegistry reg;

#if WITHXOSD != 0
  auto_ptr<PBBView> xosd_view (new PBBXosd () );
  xosd_view->Register (&reg);
#endif
  
#if WITHEVAS != 0
  auto_ptr<PBBView> evas_view (new PBBEvas () );
  evas_view->Register (&reg);
#endif
  
  std::string theme = arg_theme.Get();
  
  PBBRegistry::pair* reg_pair = reg.Query (theme);
  
  if (!reg_pair) {
    std::cout << "Theme " << theme << " not found." << std::endl;
    return 1;
  }
  
  if (arg_list.Get() == true) {
    std::cout << "The detected themes are: " << std::endl;
    for (PBBRegistry::iterator it = reg.begin (); it != reg.end (); ++it) {
      std::cout << "  " << it->first << std::endl;
    }
    return 0;
  }
  
  PBBView* used_view = reg_pair->first;
  used_view->Init (reg_pair->second);
  
  PBBInterface pbb_interface;
  pbb_interface.Init ();
  
  used_view->Main (&pbb_interface, arg_nodemo.Get());
  
  cout << "Normal termination ..." << endl;
  return 0;
}
