
/*  macosd - a pbbuttonsd client to display the pbbuttonsd events
 *           (and later maybe some other events - like Evkeyd, too)
 *  Copyright (C) 2003 Ren Rebe <rene@rocklinux.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#include <iostream>

#include "PBB.hh"

bool PBBInterface::client_exit_requested = false;

PBBInterface::PBBInterface () {
  
  // first register the TRAP handler
  signal (SIGINT, sig_trap);
  init_libpbb ();
}

PBBInterface::~PBBInterface () {
  ipc_exit ();
}

bool PBBInterface::Init ()
{
  using namespace std;
  
  int rc = ipc_init (LIBMODE_CLIENT, 1);
  if (rc != 0) {
    switch (rc) {
    case E_NOSERVER:
      cerr << "Server message port not found." << endl;
      break;
    case E_REGISTER:
      cerr << "Client list full." << endl;
      break;
    case E_MSGPORT:
      cerr << "Can't create message port for client." << endl;
      break;
    default:
      cerr << "Unknown error." << endl;
      break;
    }
    return false;
  }
  return true;
}

bool PBBInterface::NextMessage (PBBView* view)
{
  char* msgbuffer[100];
  struct tagitem* taglist;
  int timerem;
  
  using namespace std;
  
  struct pbbmessage* msg = (struct pbbmessage*) msgbuffer;
  
  if ((ipc_receive (msgbuffer, sizeof(msgbuffer))) == 0) {
    
    switch (msg->action) {
    case REGFAILED:
      cerr << "ERROR: Can't register client at server." << endl;
      return false;
      break;
      
    case CLIENTEXIT:
      client_exit_requested = true;
      return false;
      break;
      
    case CHANGEVALUE:
      taglist = msg->taglist;
      
      while (taglist->tag != TAG_END) {
	switch (taglist->tag) {
	case TAG_BRIGHTNESS:
	  view->BrightnessChange (taglist->data);
	  break;
	  
	case TAG_VOLUME:
	  if (taglist->data == -1)
	    view->NoAudio ();
	  else
	    view->AudioVolumeChange (taglist->data);
	  break;
	  
	case TAG_MUTE:
	  if (taglist->data == -1)
	    view->NoAudio ();
	  else
	    view->AudioMuted ();
	  break;
	  
	case TAG_TPMODE:
	  view->TrackpadModeChange (taglist->data);
	  break;
	}
	taglist++;
      }
      break;
      
    case WARNING:
      taglist = msg->taglist;
      while (taglist->tag != TAG_END) {
	switch (taglist->tag) {
	case TAG_CURRENTBWL:
	  // timerem = tagfind (msg->taglist, TAG_TIMEREMAINING, -1);
	  // update_interface(md, POPUP_BATLOW, taglist->data, timerem);
	  
	  view->BatteryLevelChange (taglist->data);
	  break;
	  
	case TAG_TIMEREMAINING:
	  view->MinutesRemaining (taglist->data);
	  break;
	  
	case TAG_SLEEPINSECONDS:
	  view->SecondsToSleep (taglist->data);
	  break;
	}
	++taglist;
      }
      break;
    }
  }
  return false;
}

bool PBBInterface::ClientExitRequested () {
  return client_exit_requested;
}

void PBBInterface::sig_trap (int) {
  client_exit_requested = true;
  // sigreturn;
}
