
/*  macosd - a pbbuttonsd client to display the pbbuttonsd events
 *           (and later maybe some other events - like Evkeyd, too)
 *  Copyright (C) 2003 Ren Rebe <rene@rocklinux.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __PBB_HH__
#define __PBB_HH__

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#include <iostream>
#include <string>
#include <sstream>
#include <map>

#include <xosd.h>

extern "C" {
#define template tmpl
#include <pbb.h>
#undef template
}

template <typename BASE, typename KEY, typename PRIV>
class Registry {
 public:
  
  typedef std::pair<BASE, PRIV> pair;
  typedef std::map<KEY, pair> container;
  typedef typename container::iterator iterator;
  
  bool Register (BASE base, const KEY& key, const PRIV& priv) {
    iterator it = reg.find (key);
    
    if (it == reg.end () ) {
      // std::cout << "Adding: key: " << key << ", priv: " << priv << std::endl;
      pair new_pair (base, priv);
      reg [key] = new_pair;
      return true;
    }
    std::cout << "Warning: Key " << key << " already present!" << std::endl;
    return false;
  }
  
  pair* Query (const KEY& key) {
    iterator it = reg.find (key);
    
    if (it == reg.end () ) {
      std::cout << "Warning: Key " << key << " not available!" << std::endl;
      return (pair*) 0;
    }
    return &(it->second);
  }
  
  iterator begin () {
    return reg.begin();
  }
  iterator end () {
    return reg.end ();
  }
  
 protected:
  
  container reg;
};

class PBBInterface; // forward
class PBBView; // forward

typedef Registry<PBBView*, std::string, std::string> PBBRegistry;

class PBBView {
public:
  
  PBBView () {};
  virtual ~PBBView () {};
  
  virtual void Register (PBBRegistry* reg) = 0;
  virtual bool Init (const std::string& priv) = 0;
  
  // the PBBView'ers main loop
  virtual void Main (PBBInterface* interface, bool nodemo) = 0;
  
  virtual void BrightnessChange (int i) = 0;
  virtual void AudioVolumeChange (int i) = 0;
  virtual void NoAudio () = 0;
  virtual void AudioMuted () = 0;
  virtual void TrackpadModeChange (int i) = 0;
  virtual void BatteryLevelChange (int i) = 0;
  virtual void MinutesRemaining (int i) = 0;
  virtual void SecondsToSleep (int i) = 0;
};


class PBBInterface {
public:
  PBBInterface ();
  ~PBBInterface ();

  bool Init ();
  bool NextMessage (PBBView* view);
  
  bool ClientExitRequested ();

 private:
  
  static void sig_trap (int);
  static bool client_exit_requested;
};

#endif // __PBB_HH__
