
#include <X11/Xutil.h>

#include "Evas.h"
#include "Evas_Engine_Software_X11.h"

// e.g. usleep
#include <unistd.h>

// int types
#include <inttypes.h>

// sin, cos, ceil M_PI ...
#include <math.h>

#include <iostream>
#include <sstream>
#include <fstream>

#include "DirIterator.hh"

#include "X11Helper.hh"
#include "EvasHelper.hh"

#include "PBBEvas.hh"

#include "config.h" // for gtkpbbuttons prefix (for theme loading) and version

double GetTime ()
{
  struct timeval timev;
  
  gettimeofday (&timev, NULL);
  return (double)timev.tv_sec + (((double)timev.tv_usec) / 1000000);
}

PBBEvas::PBBEvas ()
  : config_layout("layout"),
    config_bgimage("bgimage"),
    config_bgalpha("bgalpha"),
    config_pbarempty("pbarempty"),
    config_pbarfull("pbarfull"),
    config_pbarsteps("pbarsteps"),
    
    config_brightness_active("brightness.active"),
    config_brightness_image("brightness.image"),
    
    config_volume_active("volume.active"),
    config_volume_image("volume.image"),
    config_volume_sample("volume.sample"),
    
    config_mute_image("mute.image"),
    
    config_noaudio_image("noaudio.image"),
    
    config_mouse_active("mouse.active"),
    config_mouse_image("mouse.image"),
    config_mouse_mode0("mouse.mode0"),
    config_mouse_mode1("mouse.mode1"),
    config_mouse_mode2("mouse.mode2"),
    config_mouse_mode3("mouse.mode3"),
    
    config_battery_active("battery.active"),
    config_battery_image("battery.image"),
    config_battery_sample("battery.sample"),
    
    config_sleep_active("sleep.active"),
    config_sleep_image("sleep.image")
{
  config = new Serializer;

  config->Add (&config_layout);
  config->Add (&config_bgimage);
  config->Add (&config_bgalpha);
  config->Add (&config_pbarempty);
  config->Add (&config_pbarfull);
  config->Add (&config_pbarsteps);
  
  config->Add (&config_brightness_active);
  config->Add (&config_brightness_image);
    
  config->Add (&config_volume_active);
  config->Add (&config_volume_image);
  config->Add (&config_volume_sample);
  
  config->Add (&config_mute_image);
  
  config->Add (&config_noaudio_image);
  
  config->Add (&config_mouse_active);
  config->Add (&config_mouse_image);
  config->Add (&config_mouse_mode0);
  config->Add (&config_mouse_mode1);
  config->Add (&config_mouse_mode2);
  config->Add (&config_mouse_mode3);
  
  config->Add (&config_battery_active);
  config->Add (&config_battery_image);
  config->Add (&config_battery_sample);
  
  config->Add (&config_sleep_active);
  config->Add (&config_sleep_image);
  
  evas = 0;
}

PBBEvas::~PBBEvas ()
{
  if (evas) {
    delete info_text;
    
    delete progress_full;
    delete progress_empty;
    delete sub_image;
    delete main_image;
    delete bgr_image;
    delete x11_image;
    
    delete evas;
  }
  
  delete config;
}

void PBBEvas::Register (PBBRegistry* reg) {
  
  // directories to scan
  std::vector<std::string> dirs;
  dirs.push_back (GTKPBBUTTONS_PREFIX"/share/gtkpbbuttons/themes/");
  dirs.push_back (DATADIR"/macosd/themes/");
  
  // create a list of themes ...
  for (std::vector<std::string>::iterator it = dirs.begin ();
       it != dirs.end (); ++it)
    {
      const std::string& dirname = *it;
      
      Utility::DirList dir (dirname);
      Utility::DirList::Iterator it, it_end;
      
      it = dir.Begin ();
      it_end = dir.End ();
      
      while  (it != it_end) {
	// std::cout << "PBBEvas: Testing: " << *it << std::endl;
	std::string themedirname = dirname + *it + "/";
	
	std::ifstream file ( (themedirname + "theme.desc").c_str() );
	config->Read (file, false); // verbose try parse pass
	
	if (config_layout.Get() == "macosx") {
	  // std::cout << "registering..." << std::endl;
	  reg->Register (this, *it, themedirname);
	}
	++ it;
      }
    }
}

bool PBBEvas::Init (const std::string& priv)
{
  themedir = priv;
  win_w = win_h = 250;
  is_shown = false;
  x11_valid = false;
  
  // yeah - after all this is just an improved C ... :-(
  std::ifstream file (std::string (themedir + "theme.desc").c_str() );
  
  config->Read (file);
  
  // TODO: move to the X11Helper ...
  
  XSetWindowAttributes attr;
  XClassHint           chint;
  XSizeHints           szhints;
  
  disp = XOpenDisplay (NULL);
  if (!disp) {
    std::cout << "Error: cannot open display.\n" << std::endl;
    return false;
  }

  attr.backing_store = NotUseful;
  attr.colormap = DefaultColormap(disp, DefaultScreen(disp));
  attr.border_pixel = 0;
  attr.background_pixmap = None;
  attr.event_mask =
    ExposureMask | ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
    StructureNotifyMask;
  attr.bit_gravity = ForgetGravity;
  attr.override_redirect = 1;
  
  win = XCreateWindow (disp, DefaultRootWindow(disp),
		       (X11Helper::WindowWidth(disp, 0) - 250) / 2,
		       (X11Helper::WindowHeight(disp, 0) - 250) / 2,
		       win_w, win_h,
		       0,
		       DefaultDepth(disp, DefaultScreen(disp)),
		       InputOutput,
		       DefaultVisual(disp, DefaultScreen(disp)),
		       CWOverrideRedirect | 
		       CWBackingStore | CWColormap |
		       CWBackPixmap | CWBorderPixel |
		       CWBitGravity | CWEventMask, &attr);

  XStoreName (disp, win, "MacOSD");
  chint.res_name = "MacOSD";
  chint.res_class = "Main";
  XSetClassHint (disp, win, &chint);
  
#if 0
  szhints.flags = PMinSize | PMaxSize | PSize | USSize;
  szhints.min_width = szhints.max_width = win_w;
  szhints.min_height = szhints.max_height = win_h;
  XSetWMNormalHints(disp, win, &szhints);
#endif

  X11Helper::WindowStayOnTop (disp, win);
  
  XSync(disp, False);
  
  evas = new EvasCanvas ();
  
  evas->OutputMethod ("software_x11");
  evas->OutputSize (win_w, win_h);
  evas->OutputViewport (0, 0, win_w, win_h);
  
  Evas_Engine_Info_Software_X11 *einfo;
  
  einfo = (Evas_Engine_Info_Software_X11*) evas->EngineInfo ();
  
  /* the following is specific to the engine */
  einfo->info.display = disp;
  einfo->info.visual = DefaultVisual (disp, DefaultScreen(disp));
  einfo->info.colormap = DefaultColormap (disp, DefaultScreen(disp));
  einfo->info.drawable = win;
  einfo->info.depth = DefaultDepth (disp, DefaultScreen(disp));
  einfo->info.rotation = 0;
  einfo->info.debug = 0;
  
  evas->EngineInfo ( (Evas_Engine_Info*)einfo );
  
  /* Setup */
  evas->FontPathPrepend ("/usr/X11/lib/X11/fonts/TTF/");
  evas->FontPathPrepend ("/usr/X11/lib/X11/fonts/TrueType/");
  evas->FontPathPrepend ("/opt/e17/share/evas/data/");
  
  if (false) {
    evas->ImageCache (1024 * 1024);
    evas->FontCache (256 * 1024);
  }
  else {
    evas->ImageCache (0);
    evas->FontCache (0);
  }
  
  x11_image = new EvasImage (*evas);
  bgr_image = new EvasImage (*evas);
  main_image = new EvasImage (*evas);
  sub_image = new EvasImage (*evas);
  progress_empty = new EvasImage (*evas);
  progress_full = new EvasImage (*evas);
  
  info_text = new EvasText (*evas);
  
  // config / preload images ...
  main_image->Layer (5);
  sub_image->Layer (5);

  bgr_image->SmartLoad (themedir + config_bgimage.Get());
  bgr_image->ApplyAlpha (config_bgalpha.Get());
  bgr_image->Layer (0);
  bgr_image->Show ();
  
  progress_empty->SmartLoad (themedir + config_pbarempty.Get());
  
  progress_full->SmartLoad (themedir + config_pbarfull.Get());
  
  progress_steps = config_pbarsteps.Get();
  progress_width = progress_full->ImageWidth ();
  
  progress_empty->Layer (5);
  progress_full->Layer (5);

  info_text->Font ("Vera", 14);
  info_text->Text ("some_info_tb_set");
  info_text->Move (20, win_h - 30);
  info_text->Layer (5);
  info_text->Color (0xFF, 0xFF, 0xFF, 240);
  
  // create some demo stuff ;-)
  EvasImage* t_image;
  bool finished = false;
  
  for (int i = 0; !finished; ++i) {
    std::string file;
    
    switch (i) {
    case 0:
      file = config_mouse_image.Get();
      break;
    case 1:
      file = config_mouse_mode0.Get();
      break;
    case 2:
      file = config_mouse_mode1.Get();
      break;
    case 3:
      file = config_mouse_mode2.Get();
      break;
    case 4:
      file = config_mouse_mode3.Get();
      break;
    case 5: 
      file = config_brightness_image.Get();
      break;
    case 6:
      file = config_volume_image.Get();
      break;
    case 7:
      file = config_mute_image.Get();
      break;
    case 8:
      file = config_noaudio_image.Get();
      break;
    case 9:
      file = config_battery_image.Get();
      break;
    case 10:
      file = config_sleep_image.Get();
      break;
    default:
      finished = true;
    }
    if (file != "") {
      t_image = new EvasImage (*evas);
      t_image->SmartLoad (themedir + file);
      t_image->ApplyAlpha (200);
      t_image->Layer (20);
      t_image->Show ();
      demo_images.push_back (t_image);
      demo_managed_images.push_back (t_image);
    }
  }
  
  demo_images.push_back (progress_empty);
  demo_images.push_back (progress_full);
  
  EvasText* t_text;
  finished = false;
  for (int i = 0; !finished; ++i) {
    int height = height; // silence GCC
    std::string text;
    
    switch (i) {
    case 0:
      height = 12;
      text = "© 2003, 2004 René Rebe";
      break;
    case 1:
      height = 14;
      text = "MacOSD " VERSION " GPL";
      break;
    default:
      finished = true;
    }
    if (text != "") {
      
      EvasText* t_text = new EvasText (*evas);
      t_text->Font ("Vera", height);
      t_text->Text (text);
      t_text->Layer (30);
      t_text->Color (0, 0, 0, 200);
      t_text->Show ();
      demo_texts.push_back (t_text);
      demo_managed_texts.push_back (t_text);
    }
  }
  
  t_text = new EvasText (*evas);
  t_text->Font ("Vera", 10);
  t_text->Text ("");
  t_text->Layer (50);
  t_text->Color (0, 0, 0, 200);
  t_text->Show ();
  t_text->Move (8, win_h - t_text->Height () - 8);
  fps_text = t_text;
  demo_managed_texts.push_back (t_text);
  
  return true;
}

void PBBEvas::Main (PBBInterface* interface, bool nodemo)
{
  int frame = 0;
  bool demo_running = true;
  bool demo_1st_run = true;
  
  while (!interface->ClientExitRequested () )
    {
      double t;
      
      // 1st: obtain PBB events
      if (!demo_running) {
	while (interface->NextMessage (this));
	t = GetTime () - start_time;
      }
      else {
	if (!nodemo && demo_1st_run) {
	  std::cout << "starting demo ..." << std::endl;
	  main_image->Hide ();
	  StartShow ();
	  demo_1st_run = false;
	  t = 0.0;
	}
	else {
	  t = GetTime () - start_time;
	}

	// end demo?
	if (nodemo || t > 5.0) {
	  std::cout << "ending demo ..." << std::endl;
	  StopShow (true);
	  main_image->Show ();
	  
	  demo_running = false;
	  
	  while (!demo_managed_images.empty () ) {
	    delete demo_managed_images.back ();
	    demo_managed_images.pop_back ();
	    demo_images.clear ();
	  }
	  while (!demo_managed_texts.empty () ) {
	    delete demo_managed_texts.back ();
	    demo_managed_texts.pop_back ();
	    demo_texts.clear ();
	  }
	}
	else {
	  // move the objects around
	  
	  int n;
	  Evas_Coord x, y;
	  double     v;
	  
	  // text
	  n = demo_texts.size ();
	  for (int i = 0; i < n; ++i)
	    {
	      v = (t  / 2.5 - 1.0) * M_PI / -2.0;
	      y = sin(v) * (double)((win_h + 32) / 2) -
		((i + 1) / 2) * demo_texts[i]->Height ();
	      demo_texts[i]->Move ((win_h - demo_texts[i]->Width () ) / 2,
				   win_h/2 + y);
	      
	    }
	  
	  // images
	  n = demo_images.size ();
	  for (int i = 0; i < n; ++i)
	    {
	      v = (1. - t  / 5.0) * M_PI * 2;
	      x = cos(v * (double)(i+1) / 3) * ((win_w - 48) / 2);
	      y = sin(v * (double)(i+1) / 3) * ((win_h - 48) / 2);
	      demo_images[i]->Move (win_h/2 + x - 24,
				    win_h/2 + y - 24);
	      
	      if (false && i & 0x1)
		{
		  v = (((t) / 6.0) * M_PI * 2) * 2;
		  x = (cos(v * (double)i * 2) * 48) + 50;
		  y = (sin(v * (double)i *5) * 48) + 50;
		  demo_images[i]->Resize (x, y);
		}
	    }
	  
	  // fps counter
	  if (frame > 0 && t > 0.0) {
	    std::stringstream text;
	    text << "fps: " << (int) (frame / t);
	    fps_text->Text (text.str() );;
	  }
	}
      }
      
      // 2nd: process X11 events ...
      // TODO: move into X11 Helper ...
      
      XEvent ev;
      while (XCheckMaskEvent (disp,
			      ExposureMask |
			      StructureNotifyMask |
			      KeyPressMask |
			      KeyReleaseMask |
			      ButtonPressMask |
			      ButtonReleaseMask | PointerMotionMask, &ev))
	{
	  switch (ev.type)
	    {
	    case ButtonPress:
	      evas->EventFeedMouseMove (ev.xbutton.x, ev.xbutton.y);
 	      evas->EventFeedMouseDown (ev.xbutton.button);
	      break;
	    case ButtonRelease:
	      evas->EventFeedMouseMove (ev.xbutton.x, ev.xbutton.y);
 	      evas->EventFeedMouseUp (ev.xbutton.button);
	      break;
	    case MotionNotify:
	      evas->EventFeedMouseMove (ev.xmotion.x, ev.xmotion.y);
	      break;
	    case Expose:
	      evas->DamageRectangleAdd (ev.xexpose.x,
					ev.xexpose.y,
					ev.xexpose.width,
					ev.xexpose.height);
	      break;
	    case ConfigureNotify:
	      evas->OutputSize (ev.xconfigure.width,
				ev.xconfigure.height);
	      break;
	    default:
	      break;
	    }
	}
      
      evas->Render ();
      XFlush (disp);
      
      // stop animation?
      if (!demo_running && is_shown && t > 1.25) {
	std::cout << "Stopping show ..." << std::endl;
	StopShow (true);
      }
      
      if (demo_running)
	usleep (2500);
      else
	usleep (25000);
      ++ frame;
    }
}
  
void PBBEvas::BrightnessChange (int i)
{
  StopShow ();
  
  main_image->SmartLoad (themedir + config_brightness_image.Get());
  main_image->Show ();
  main_image->Move (0,0);
  
  SetProgress ((double)i / 15);
  
  StartShow ();
}

void PBBEvas::AudioVolumeChange (int i)
{
  StopShow ();
  
  main_image->SmartLoad (themedir + config_volume_image.Get());
  main_image->Show ();
  SetProgress ((double)i / 100);
  
  StartShow ();
}

void PBBEvas::NoAudio ()
{
  StopShow ();
  
  main_image->SmartLoad (themedir + config_noaudio_image.Get());
  main_image->Show ();
  SetProgress (0.0);
  
  StartShow ();
}

void PBBEvas::AudioMuted ()
{
  StopShow ();
  main_image->SmartLoad (themedir + config_mute_image.Get());
  main_image->Show ();
  SetProgress (0.0);
  
  StartShow ();
}

void PBBEvas::TrackpadModeChange (int i)
{
  StopShow ();
  
  main_image->SmartLoad (themedir + config_mouse_image.Get());
  main_image->Show ();
  
  switch (i) {
  case 0:
    sub_image->SmartLoad (themedir + config_mouse_mode0.Get());
    break;
  case 1:
    sub_image->SmartLoad (themedir + config_mouse_mode1.Get());
    break;
  case 2:
    sub_image->SmartLoad (themedir + config_mouse_mode2.Get());
    break;
  case 3:
    sub_image->SmartLoad (themedir + config_mouse_mode3.Get());
    break;
  default:
    std::cerr << "Error: Invalid touchpad mode received: " << i << std::endl;
  }
  sub_image->Show ();
  
  StartShow ();
}

void PBBEvas::BatteryLevelChange (int i)
{
  StopShow ();
  main_image->SmartLoad (themedir + config_battery_image.Get());
  main_image->Show ();
  StartShow ();
}

void PBBEvas::MinutesRemaining (int i)
{
  StopShow ();
  main_image->SmartLoad (themedir + config_battery_image.Get());
  main_image->Show ();
  
  std::stringstream text;
  text << i << " minutes remaining";

  info_text->Text (text.str());
  info_text->Show ();
  
  StartShow ();
}

void PBBEvas::SecondsToSleep (int i)
{
  StopShow ();
  main_image->SmartLoad (themedir + config_sleep_image.Get());
  main_image->Show ();

  std::stringstream text;
  text << i << " seconds remaining";

  info_text->Text (text.str());
  info_text->Show ();
  
  StartShow ();
}

void PBBEvas::UpdateX11Image ()
{
  // the captured screen (for the damn usual X11 pseudo transparency) ...
  X11Helper::WindowToEvasImage (x11_image->c_obj (), disp, 0,
				(X11Helper::WindowWidth(disp, 0) - 250) / 2,
				(X11Helper::WindowHeight(disp, 0) - 250) / 2,
				win_w, win_h);
  x11_image->Move (0, 0);
  x11_image->Resize (win_w, win_h);
  x11_image->Layer (-5);
  x11_image->Show ();
}

void PBBEvas::SetProgress (double i)
{
  // re-layout
  int tx = (win_w - progress_empty->ImageWidth () ) / 2;
  int ty = (win_h - progress_empty->ImageHeight () ) - 20;
  
  progress_empty->Move (tx, ty);
  progress_full->Move (tx, ty);

  // scale for progress
  i = ceil (i * progress_steps) / progress_steps;
  int width = (int) (progress_width * i);

  progress_full->Resize (width, progress_full->ImageHeight () );

  progress_empty->Show ();
  progress_full->Show ();
}

void PBBEvas::StopShow (bool outdate_x11) {
  if (is_shown) {
    info_text->Hide ();
    sub_image->Hide ();
    progress_empty->Hide ();
    progress_full->Hide ();
    
    if (outdate_x11) {
      XUnmapWindow (disp, win);
      
      is_shown = false;
      x11_valid = !outdate_x11;
    }
  }
}

void PBBEvas::StartShow () {
  if (!x11_valid) {
    UpdateX11Image ();
    XMapWindow (disp, win);
    x11_valid = true;
  }
  
  XRaiseWindow(disp, win);
  
  // layouting ... - a real CSS box style layouting system will
  // be added later ...
  
  int x, y;
  
  x = (win_w - main_image->ImageWidth () ) / 2;
  y = 30; // (win_h - main_image->ImageHeight () ) / 2;
  main_image->Move (x, y);

  x = (win_w - sub_image->ImageWidth () ) / 2;
  y = (win_h - sub_image->ImageHeight () ) - 20;
  sub_image->Move (x, y);
  
  x = (win_w - (int)info_text->Width () ) / 2;
  y = (win_h - (int)info_text->Height () ) - 20;
  info_text->Move (x, y);
  
  start_time = GetTime ();
  is_shown = true;
}
