
#ifndef __PBBXEVAS_HH__
#define __PBBXEVAS_HH__

#include <X11/Xlib.h>
#include <Evas.h>

#include <string>
#include <vector>

#include "EvasHelper.hh"
#include "PBB.hh"

#include "Storage.hh"

using namespace Utility;

class PBBEvas : public PBBView
{
 public:	
  PBBEvas ();
  virtual ~PBBEvas ();
  
  virtual void Register (PBBRegistry* reg);
  virtual bool Init (const std::string& priv);
  
  virtual void BrightnessChange (int i);
  virtual void AudioVolumeChange (int i);
  virtual void NoAudio ();
  virtual void AudioMuted ();
  virtual void TrackpadModeChange (int i);
  virtual void BatteryLevelChange (int i);
  virtual void MinutesRemaining (int i);
  virtual void SecondsToSleep (int i);
  
  virtual void Main (PBBInterface* interface, bool nodemo);
  
 private:

  void UpdateX11Image ();
  void SetProgress (double i);
  
  void StartShow ();
  void StopShow (bool outdate_x11 = false);
  
  // X11 stuff
  Display* disp;
  Window  win;
  
  bool is_shown;
  bool x11_valid;
  double start_time;
  
  // the window size - defined by the background image ...
  int win_w;
  int win_h;
  
  EvasCanvas* evas;
  
  EvasImage* x11_image;
  EvasImage* bgr_image;
  EvasImage* main_image;
  EvasImage* sub_image;
  EvasImage* progress_empty;
  EvasImage* progress_full;
  
  EvasText* info_text;
  
  int progress_steps;
  int progress_width;
  
  std::vector<EvasImage*> demo_images; // images to be animated
  std::vector<EvasImage*> demo_managed_images; // imates to be deleted
  
  EvasText* fps_text;
  std::vector<EvasText*> demo_texts; // dito for the texts
  std::vector<EvasText*> demo_managed_texts; // dito for the texts
  
  // theming
  std::string themedir;
  Serializer* config;
  
  Storage <std::string> config_layout;
  Storage <std::string> config_bgimage;
  Storage <int> config_bgalpha;
  Storage <std::string> config_pbarempty;
  Storage <std::string> config_pbarfull;
  Storage <int> config_pbarsteps;
  
  Storage <bool> config_brightness_active;
  Storage <std::string> config_brightness_image;
  
  Storage <bool> config_volume_active;
  Storage <std::string> config_volume_image;
  Storage <std::string> config_volume_sample;
  
  Storage <std::string> config_mute_image;
  
  Storage <std::string> config_noaudio_image;
  
  Storage <bool> config_mouse_active;
  Storage <std::string> config_mouse_image;
  Storage <std::string> config_mouse_mode0;
  Storage <std::string> config_mouse_mode1;
  Storage <std::string> config_mouse_mode2;
  Storage <std::string> config_mouse_mode3;
  
  Storage <bool>  config_battery_active;
  Storage <std::string> config_battery_image;
  Storage <std::string> config_battery_sample;
  
  Storage <bool> config_sleep_active;
  Storage <std::string> config_sleep_image;
};

#endif // __PBBEVAS_HH__

