
#include <xosd.h>

#include <sstream>

#include "PBBXosd.hh"

#include "config.h" // just vor the version ...

PBBXosd::PBBXosd () {
  osd = xosd_create (2); // FIXME: ?

  if (!osd) {
    PrintError ();
  }
};

PBBXosd::~PBBXosd () {
  if (0 != xosd_destroy (osd)) {
     PrintError();
  }
};

void PBBXosd::Register (PBBRegistry* reg) {
  reg->Register (this, "classic", "");
}

bool PBBXosd::Init (const std::string& priv) {
  
  if (priv != "")
    std::cout << "Warning PBBXosd got a priv string?" << std::endl;
  
  if (0 != xosd_set_font(osd, "-*-helvetica-*-r-*-*-34-*-*-*-*-*-*-*") ) {
    if (0 != xosd_set_font(osd, "-*-*-*-r-*-*-34-*-*-*-*-*-*-*") ) {
      PrintError ();
      return 1;
    }
  }

  xosd_set_colour (osd, "white");

  if (0 != xosd_set_timeout(osd, 2)) {
    PrintError();
  }

  if (0 != xosd_set_pos (osd, XOSD_middle)) {
    PrintError();
  }

  xosd_set_shadow_offset (osd, 2);
  xosd_set_outline_colour(osd, "black");

  xosd_set_outline_offset (osd, 1);

  if (0 != xosd_set_align (osd, XOSD_center)) {
    PrintError();
  }

  xosd_set_bar_length (osd, 24);
  xosd_set_vertical_offset (osd, 0);
  xosd_set_horizontal_offset (osd, 0);
  
  // display some info
  ShowText (0, "Mac OSD - Rene Rebe");
  ShowText (1, "GPL - " VERSION);
  WaitNotDisplayed ();
  return true;
}

void PBBXosd::Main (PBBInterface* interface, bool nodemo) {
  
  while (!interface->ClientExitRequested ()) {
    if (!interface->NextMessage (this) )
      usleep (25000);
  }
}

void PBBXosd::BrightnessChange (int i) {
  ShowText       (0, "Brightness level");
  ShowPercentage (1, i * 100 / 15);

}

void PBBXosd::AudioVolumeChange (int i) {
  ShowText       (0, "Audio output level");
  ShowPercentage (1, i);
}

void PBBXosd::NoAudio () {
  ShowText (0, "No audio");
  ShowText (1, "");
}

void PBBXosd::AudioMuted () {
  ShowText (0, "Audio muted");
  ShowText (1, "");
}

void PBBXosd::TrackpadModeChange (int i) {
  ShowText (0, "Audio muted");
  ShowText (1, "");
}

void PBBXosd::BatteryLevelChange (int i) {
  std::stringstream text;
  text << "Battery level: " << i;
  ShowText (0, text.str () );
  ShowText (1, "");
}

void PBBXosd::MinutesRemaining (int i) {
  std::stringstream text;
  text << i << " minutes remaining!";
  ShowText (0, text.str () );
  ShowText (1, "");
}

void PBBXosd::SecondsToSleep (int i) {
  std::stringstream text;
  text << "Sleeping in " << i << " seconds!";
  ShowText (0, text.str () );
  ShowText (1, "");
}

// private:

void PBBXosd::PrintError () {
  std::cerr << "ERROR: " << xosd_error << std::endl;
}

bool PBBXosd::ShowText (int row, std::string text) {
  if (-1 == xosd_display (osd, row, XOSD_string, text.c_str())) {
    PrintError();
    return false;
  }
  return true;
}

bool PBBXosd::ShowPercentage (int row, int value) {
  if (-1 == xosd_display (osd, row, XOSD_percentage, value)) {
    PrintError ();
    return false;
  }
  return true;
}

bool PBBXosd::IsDisplayed () {
  return xosd_is_onscreen (osd) == 0;
}


void PBBXosd::WaitNotDisplayed () {
  xosd_wait_until_no_display (osd);
}
