/*
 * --- GSMP-COPYRIGHT-NOTE-BEGIN ---
 * 
 * This copyright note is auto-generated by ./scripts/Create-CopyPatch.
 * Please add additional copyright information _after_ the line containing
 * the GSMP-COPYRIGHT-NOTE-END tag. Otherwise it might get removed by
 * the ./scripts/Create-CopyPatch script. Do not edit this copyright text!
 * 
 * GSMP: utility/src/File.cc
 * General Sound Manipulation Program is Copyright (C) 2000 - 2004
 *   Valentin Ziegler and Ren Rebe
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2. A copy of the GNU General
 * Public License can be found in the file LICENSE.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 * ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * --- GSMP-COPYRIGHT-NOTE-END ---
 */

#include "File.hh"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <iostream>

Utility::File::File (const std::string& str) {
  filename = str;
  c_stat_valid = false;
}

Utility::File::~File (){
}

const std::string& Utility::File::Basename ()
{
  // we always have a basename - a leading dot forms a hidden Unix file ....
  if (basename == "")
    updateBaseExt ();
  return basename;
}

const std::string& Utility::File::Extension ()
{
  // this is slightly unoptimal if we really do not have an extension
  // but since this is rare we keep this simple
  if (extension == "")
    updateBaseExt ();
  return extension;
}

bool Utility::File::IsDirectory () {
  return updateStat() && S_ISDIR(c_stat.st_mode);
}

bool Utility::File::IsSymlink () {
  return updateStat() && S_ISLNK(c_stat.st_mode);
}

bool Utility::File::IsFile () {
  return updateStat() && S_ISREG(c_stat.st_mode);
}

bool Utility::File::updateStat () {
  if (c_stat_valid)
    return true;
  
  return stat (filename.c_str(), &c_stat) == 0;
}

void Utility::File::updateBaseExt () {
  // parse the filename extension
  std::string::size_type idx = filename.rfind ('.');
  
  // ignore leading dots - they mark hidden Unix files ...
  if (idx && idx != std::string::npos) {
    basename = filename.substr (0, idx);
    extension = filename.substr (idx + 1);
  }
  else {
    basename = filename;
    extension.clear (); // slightly redundant - but just to be sure
  }
}
